# SPDX-FileCopyrightText: 2025 Deutsche Telekom AG (opensource@telekom.de)
#
# SPDX-License-Identifier: Apache-2.0

from .path import PathToFolderWithBaseModels

__all__=["PathToFolderWithBaseModels"]
