# SPDX-FileCopyrightText: 2025 Deutsche Telekom AG (opensource@telekom.de)
#
# SPDX-License-Identifier: Apache-2.0

#pragma: no cover
#pylint: skip-file
"""CLI Entry"""
from wurzel.cli._main import main as _main

if __name__ == '__main__':
    _main()
